@@ -1,36 +1,31 @@
-/* vim:set shiftwidth=4 ts=8: */
+/**
+ * @file
+ * @brief graphs, nodes and edges info: Agraphinfo_t, Agnodeinfo_t and Agedgeinfo_t
+ * @ingroup public_apis
+ * @ingroup common_render
+ */
 
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef GV_TYPES_H
-#define GV_TYPES_H
+#pragma once
 
 /* Define if you want CGRAPH */
 #define WITH_CGRAPH 1
 
+#include <stdbool.h>
+#include <stddef.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <assert.h>
 #include <signal.h>
-
-typedef unsigned char boolean;
-#ifndef NOT
-#define NOT(v) (!(v))
-#endif
-#ifndef FALSE
-#define FALSE 0
-#endif
-#ifndef TRUE
-#define TRUE NOT(FALSE)
-#endif
-
 #include "geom.h"
 #include "gvcext.h"
 #include "pathgeom.h"
@@ -41,8 +36,6 @@ typedef unsigned char boolean;
 extern "C" {
 #endif
 
-    typedef int (*qsort_cmpf) (const void *, const void *);
-    typedef int (*bsearch_cmpf) (const void *, const void *);
     typedef struct Agraph_s graph_t;
     typedef struct Agnode_s node_t;
     typedef struct Agedge_s edge_t;
@@ -52,27 +45,16 @@ extern "C" {
 
     typedef struct htmllabel_t htmllabel_t;
 
-    typedef union inside_t {
-	struct {
-	    pointf* p;
-	    double* r;
-	} a;
-	struct {
-	    node_t* n;
-	    boxf*    bp;
-	} s;
-    } inside_t;
-
     typedef struct port {	/* internal edge endpoint specification */
 	pointf p;		/* aiming point relative to node center */
 	double theta;		/* slope in radians */
 	boxf *bp;		/* if not null, points to bbox of
 				 * rectangular area that is port target
 				 */
-	boolean	defined;        /* if true, edge has port info at this end */
-	boolean	constrained;    /* if true, constraints such as theta are set */
-	boolean clip;           /* if true, clip end to node/port shape */
-	boolean dyna;           /* if true, assign compass point dynamically */
+	bool defined;        /* if true, edge has port info at this end */
+	bool constrained;    /* if true, constraints such as theta are set */
+	bool clip;           /* if true, clip end to node/port shape */
+	bool dyna;           /* if true, assign compass point dynamically */
 	unsigned char order;	/* for mincross */
 	unsigned char side;	/* if port is on perimeter of node, this
                                  * contains the bitwise OR of the sides (TOP,
@@ -82,10 +64,10 @@ extern "C" {
     } port;
 
     typedef struct {
-	boolean(*swapEnds) (edge_t * e);	/* Should head and tail be swapped? */
-	boolean(*splineMerge) (node_t * n);	/* Is n a node in the middle of an edge? */
-	boolean ignoreSwap;                     /* Test for swapped edges if false */
-	boolean isOrtho;                        /* Orthogonal routing used */
+	bool(*swapEnds) (edge_t * e);	/* Should head and tail be swapped? */
+	bool(*splineMerge) (node_t * n);	/* Is n a node in the middle of an edge? */
+	bool ignoreSwap;                     /* Test for swapped edges if false */
+	bool isOrtho;                        /* Orthogonal routing used */
     } splineInfo;
 
     typedef struct pathend_t {
@@ -99,23 +81,23 @@ extern "C" {
     typedef struct path {	/* internal specification for an edge spline */
 	port start;
 	port end;
-	int nbox;		/* number of subdivisions */
+	size_t nbox; ///< number of subdivisions
 	boxf *boxes;		/* rectangular regions of subdivision */
 	void *data;
     } path;
 
     typedef struct bezier {
 	pointf *list;
-	int size;
-	int sflag;
-	int eflag;
+	size_t size;
+	uint32_t sflag;
+	uint32_t eflag;
 	pointf sp;
 	pointf ep;
     } bezier;
 
     typedef struct splines {
 	bezier *list;
-	int size;
+	size_t size;
 	boxf bb;
     } splines;
 
@@ -133,51 +115,71 @@ extern "C" {
 	union {
 	    struct {
 		textspan_t *span;
-		short nspans;
+		size_t nspans;
 	    } txt;
 	    htmllabel_t *html;
 	} u;
 	char valign;  /* 't' 'c' 'b' */
-	boolean set;  /* true if position is set */
-	boolean html; /* true if html label */
+	bool set;  /* true if position is set */
+	bool html; /* true if html label */
     } textlabel_t;
 
+    typedef struct {
+	bool filled: 1;
+	bool radial: 1;
+	bool rounded: 1;
+	bool diagonals: 1;
+	bool auxlabels: 1;
+	bool invisible: 1;
+	bool striped: 1;
+	bool dotted: 1;
+	bool dashed: 1;
+	bool wedged: 1;
+	bool underline: 1;
+	bool fixedshape: 1;
+	unsigned shape: 7;
+    } graphviz_polygon_style_t;
+
     typedef struct polygon_t {	/* mutable shape information for a node */
 	int regular;		/* true for symmetric shapes */
-	int peripheries;	/* number of periphery lines */
-	int sides;		/* number of sides */
+	size_t peripheries; ///< number of periphery lines
+	size_t sides; ///< number of sides
 	double orientation;	/* orientation of shape (+ve degrees) */
 	double distortion;	/* distortion factor - as in trapezium */
 	double skew;		/* skew factor - as in parallelogram */
-	int option;		/* ROUNDED, DIAGONAL corners, etc. */
+	graphviz_polygon_style_t option; ///< ROUNDED, DIAGONAL corners, etc.
 	pointf *vertices;	/* array of vertex points */
     } polygon_t;
 
+typedef union inside_t {
+  struct {
+    pointf *p;
+    double *r;
+  } a;
+  struct {
+    node_t *n;
+    boxf *bp;
+    node_t *lastn;        ///< last node argument
+    double radius;        ///< last radius seen
+    polygon_t *last_poly; ///< last seen polygon
+    size_t last;          ///< last used polygon vertex
+    size_t outp;          ///< last used outline periphery
+    double scalex, scaley, box_URx, box_URy;
+        ///< various computed sizes of aspects of the last seen polygon
+  } s;
+} inside_t;
+
     typedef struct stroke_t {	/* information about a single stroke */
 	/* we would have called it a path if that term wasn't already used */
-	int nvertices;		/* number of points in the stroke */
-	int flags;		/* stroke style flags */
+	size_t nvertices; ///< number of points in the stroke
 	pointf *vertices;	/* array of vertex points */
     } stroke_t;
 
-/* flag definitions for stroke_t */
-#define STROKE_CLOSED (1 << 0)
-#define STROKE_FILLED (1 << 1)
-#define STROKE_PENDOWN (1 << 2)
-#define STROKE_VERTICES_ALLOCATED (1 << 3)
-
-    typedef struct shape_t {	/* mutable shape information for a node */
-	int nstrokes;		/* number of strokes in array */
-	stroke_t *strokes;	/* array of strokes */
-	/* The last stroke must always be closed, but can be pen_up.
-	 * It is used as the clipping path */
-    } shape_t;
-
     typedef struct shape_functions {	/* read-only shape functions */
 	void (*initfn) (node_t *);	/* initializes shape from node u.shape_info structure */
 	void (*freefn) (node_t *);	/* frees  shape from node u.shape_info structure */
 	 port(*portfn) (node_t *, char *, char *);	/* finds aiming point and slope of port */
-	 boolean(*insidefn) (inside_t * inside_context, pointf);	/* clips incident gvc->e spline on shape of gvc->n */
+	 bool(*insidefn) (inside_t * inside_context, pointf);	/* clips incident gvc->e spline on shape of gvc->n */
 	int (*pboxfn)(node_t* n, port* p, int side, boxf rv[], int *kptr); /* finds box path to reach port */
 	void (*codefn) (GVJ_t * job, node_t * n);	/* emits graphics code for node */
     } shape_functions;
@@ -188,23 +190,12 @@ extern "C" {
 	char *name;		/* as read from graph file */
 	shape_functions *fns;
 	polygon_t *polygon;	/* base polygon info */
-	boolean usershape;
+	bool usershape;
     } shape_desc;
 
 #include "usershape.h"		/* usershapes needed by gvc */
 
-    typedef struct nodequeue {
-	node_t **store;
-	node_t **limit;
-	node_t **head;
-	node_t **tail;
-    } nodequeue;
-
-    typedef struct adjmatrix_t {
-	int nrows;
-	int ncols;
-	char *data;
-    } adjmatrix_t;
+    typedef struct adjmatrix_t adjmatrix_t;
 
     typedef struct rank_t {
 	int n;			/* number of nodes in this rank  */
@@ -215,9 +206,9 @@ extern "C" {
 	double ht2;	/* height below/above centerline    */
 	double pht1;	/* as above, but only primitive nodes   */
 	double pht2;	/* as above, but only primitive nodes   */
-	boolean candidate;	/* for transpose () */
-	boolean valid;
-	int cache_nc;		/* caches number of crossings */
+	bool candidate;	/* for transpose () */
+	bool valid;
+	int64_t cache_nc;		/* caches number of crossings */
 	adjmatrix_t *flat;
     } rank_t;
 
@@ -232,9 +223,9 @@ extern "C" {
 	pointf margin;
 	pointf page;
 	pointf size;
-	boolean filled;
-	boolean landscape;
-	boolean centered;
+	bool filled;
+	bool landscape;
+	bool centered;
 	ratio_t ratio_kind;
 	void* xdots;
 	char* id;
@@ -254,12 +245,12 @@ extern "C" {
 
     typedef struct nlist_t {
 	node_t **list;
-	int size;
+	size_t size;
     } nlist_t;
 
     typedef struct elist {
 	edge_t **list;
-	int size;
+	size_t size;
     } elist;
 
 #define GUI_STATE_ACTIVE    (1<<0)
@@ -267,13 +258,23 @@ extern "C" {
 #define GUI_STATE_VISITED   (1<<2)
 #define GUI_STATE_DELETED   (1<<3)
 
-#define elist_fastapp(item,L) do {L.list[L.size++] = item; L.list[L.size] = NULL;} while(0)
-#define elist_append(item,L)  do {L.list = ALLOC(L.size + 2,L.list,edge_t*); L.list[L.size++] = item; L.list[L.size] = NULL;} while(0)
-#define alloc_elist(n,L)      do {L.size = 0; L.list = N_NEW(n + 1,edge_t*); } while (0)
+#define elist_append(item, L)                                                  \
+  do {                                                                         \
+    L.list = gv_recalloc(L.list, L.size + 1, L.size + 2, sizeof(edge_t *));    \
+    L.list[L.size++] = item;                                                   \
+    L.list[L.size] = NULL;                                                     \
+  } while (0)
+#define alloc_elist(n, L)                                                      \
+  do {                                                                         \
+    L.size = 0;                                                                \
+    L.list = gv_calloc(n + 1, sizeof(edge_t *));                               \
+  } while (0)
 #define free_list(L)          free(L.list)
 
 typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 
+/// @addtogroup cgraph_graph
+/// @{
     typedef struct Agraphinfo_t {
 	Agrec_t hdr;
 	/* to generate code */
@@ -283,7 +284,7 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 	pointf border[4];	/* sizes of margins for graph labels */
 	unsigned char gui_state; /* Graph state for GUI ops */
 	unsigned char has_labels;
-	boolean has_images;
+	bool has_images;
 	unsigned char charset; /* input character set */
 	int rankdir;
 	double ht1; /* below and above extremal ranks */
@@ -321,15 +322,12 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 	/* connected components */
 	node_t *minset; /* set leaders */
 	node_t *maxset;	/* set leaders */
-	long n_nodes;
 	/* includes virtual */
 	int minrank;
 	int maxrank;
 
 	/* various flags */
-	boolean has_flat_edges;
-	boolean has_sourcerank;
-	boolean has_sinkrank;
+	bool has_flat_edges;
 	unsigned char	showboxes;
 	fontname_kind fontnames;		/* to override mangling in SVG */
 
@@ -341,11 +339,11 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 	/* for clusters */
 	node_t *leader;
 	node_t **rankleader;
-	boolean expanded;
+	bool expanded;
 	char installed;
 	char set_type;
 	char label_pos;
-	boolean exact_ranksep;
+	bool exact_ranksep;
 #endif
 
     } Agraphinfo_t;
@@ -359,7 +357,6 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define GD_dist(g) (((Agraphinfo_t*)AGDATA(g))->dist)
 #define GD_alg(g) (((Agraphinfo_t*)AGDATA(g))->alg)
 #define GD_border(g) (((Agraphinfo_t*)AGDATA(g))->border)
-#define GD_cl_cnt(g) (((Agraphinfo_t*)AGDATA(g))->cl_nt)
 #define GD_clust(g) (((Agraphinfo_t*)AGDATA(g))->clust)
 #define GD_dotroot(g) (((Agraphinfo_t*)AGDATA(g))->dotroot)
 #define GD_comp(g) (((Agraphinfo_t*)AGDATA(g))->comp)
@@ -371,11 +368,8 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define GD_has_labels(g) (((Agraphinfo_t*)AGDATA(g))->has_labels)
 #define GD_has_images(g) (((Agraphinfo_t*)AGDATA(g))->has_images)
 #define GD_has_flat_edges(g) (((Agraphinfo_t*)AGDATA(g))->has_flat_edges)
-#define GD_has_sourcerank(g)	(((Agraphinfo_t*)AGDATA(g))->has_sourcerank)
-#define GD_has_sinkrank(g)	(((Agraphinfo_t*)AGDATA(g))->has_sinkrank)
 #define GD_ht1(g) (((Agraphinfo_t*)AGDATA(g))->ht1)
 #define GD_ht2(g) (((Agraphinfo_t*)AGDATA(g))->ht2)
-#define GD_inleaf(g) (((Agraphinfo_t*)AGDATA(g))->inleaf)
 #define GD_installed(g) (((Agraphinfo_t*)AGDATA(g))->installed)
 #define GD_label(g) (((Agraphinfo_t*)AGDATA(g))->label)
 #define GD_leader(g) (((Agraphinfo_t*)AGDATA(g))->leader)
@@ -393,13 +387,11 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define GD_maxrep(g) (((Agraphinfo_t*)AGDATA(g))->maxrep)
 #define GD_move(g) (((Agraphinfo_t*)AGDATA(g))->move)
 #define GD_n_cluster(g) (((Agraphinfo_t*)AGDATA(g))->n_cluster)
-#define GD_n_nodes(g) (((Agraphinfo_t*)AGDATA(g))->n_nodes)
 #define GD_ndim(g) (((Agraphinfo_t*)AGDATA(g))->ndim)
 #define GD_odim(g) (((Agraphinfo_t*)AGDATA(g))->odim)
 #define GD_neato_nlist(g) (((Agraphinfo_t*)AGDATA(g))->neato_nlist)
 #define GD_nlist(g) (((Agraphinfo_t*)AGDATA(g))->nlist)
 #define GD_nodesep(g) (((Agraphinfo_t*)AGDATA(g))->nodesep)
-#define GD_outleaf(g) (((Agraphinfo_t*)AGDATA(g))->outleaf)
 #define GD_rank(g) (((Agraphinfo_t*)AGDATA(g))->rank)
 #define GD_rankleader(g) (((Agraphinfo_t*)AGDATA(g))->rankleader)
 #define GD_ranksep(g) (((Agraphinfo_t*)AGDATA(g))->ranksep)
@@ -411,7 +403,10 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define GD_spring(g) (((Agraphinfo_t*)AGDATA(g))->spring)
 #define GD_sum_t(g) (((Agraphinfo_t*)AGDATA(g))->sum_t)
 #define GD_t(g) (((Agraphinfo_t*)AGDATA(g))->t)
+/// @}
 
+/// @addtogroup cgraph_node
+/// @{
     typedef struct Agnodeinfo_t {
 	Agrec_t hdr;
 	shape_desc *shape;
@@ -423,12 +418,14 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 	double ht;
 	double lw;
 	double rw;
+	double outline_width;  /* width in points with penwidth taken into account */
+	double outline_height; /* height in points with penwidth taken into account */
 	textlabel_t *label;
 	textlabel_t *xlabel;
 	void *alg;
 	char state;
 	unsigned char gui_state; /* Node state for GUI ops */
-	boolean clustnode;
+	bool clustnode;
 
 #ifndef DOT_ONLY
 	unsigned char pinned;
@@ -440,13 +437,13 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #endif
 #ifndef NEATO_ONLY
 	unsigned char showboxes;
-	boolean  has_port;
+	bool  has_port;
 	node_t* rep;
 	node_t *set;
 
 	/* fast graph */
 	char node_type;
-	char mark;
+	size_t mark;
 	char onstack;
 	char ranktype;
 	char weight_class;
@@ -462,8 +459,6 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 	/* for union-find and collapsing nodes */
 	int UF_size;
 	node_t *UF_parent;
-	node_t *inleaf;
-	node_t *outleaf;
 
 	/* for placing nodes */
 	int rank;
@@ -504,7 +499,6 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define ND_hops(n) (((Agnodeinfo_t*)AGDATA(n))->hops)
 #define ND_ht(n) (((Agnodeinfo_t*)AGDATA(n))->ht)
 #define ND_in(n) (((Agnodeinfo_t*)AGDATA(n))->in)
-#define ND_inleaf(n) (((Agnodeinfo_t*)AGDATA(n))->inleaf)
 #define ND_label(n) (((Agnodeinfo_t*)AGDATA(n))->label)
 #define ND_xlabel(n) (((Agnodeinfo_t*)AGDATA(n))->xlabel)
 #define ND_lim(n) (((Agnodeinfo_t*)AGDATA(n))->lim)
@@ -519,7 +513,8 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define ND_order(n) (((Agnodeinfo_t*)AGDATA(n))->order)
 #define ND_other(n) (((Agnodeinfo_t*)AGDATA(n))->other)
 #define ND_out(n) (((Agnodeinfo_t*)AGDATA(n))->out)
-#define ND_outleaf(n) (((Agnodeinfo_t*)AGDATA(n))->outleaf)
+#define ND_outline_width(n) (((Agnodeinfo_t*)AGDATA(n))->outline_width)
+#define ND_outline_height(n) (((Agnodeinfo_t*)AGDATA(n))->outline_height)
 #define ND_par(n) (((Agnodeinfo_t*)AGDATA(n))->par)
 #define ND_pinned(n) (((Agnodeinfo_t*)AGDATA(n))->pinned)
 #define ND_pos(n) (((Agnodeinfo_t*)AGDATA(n))->pos)
@@ -541,7 +536,10 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define ND_width(n) (((Agnodeinfo_t*)AGDATA(n))->width)
 #define ND_xsize(n) (ND_lw(n)+ND_rw(n))
 #define ND_ysize(n) (ND_ht(n))
+/// @}
 
+/// @addtogroup cgraph_edge
+/// @{
     typedef struct Agedgeinfo_t {
 	Agrec_t hdr;
 	splines *spl;
@@ -566,13 +564,13 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #endif
 #ifndef NEATO_ONLY
 	unsigned char showboxes;
-	boolean conc_opp_flag;
+	bool conc_opp_flag;
 	short xpenalty;
 	int weight;
 	int cutvalue;
 	int tree_index;
 	short count;
-	unsigned short minlen;
+	int minlen;
 	edge_t *to_virt;
 #endif
     } Agedgeinfo_t;
@@ -603,14 +601,20 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #define ED_xpenalty(e) (((Agedgeinfo_t*)AGDATA(e))->xpenalty)
 #define ED_dist(e) (((Agedgeinfo_t*)AGDATA(e))->dist)
 #define ED_weight(e) (((Agedgeinfo_t*)AGDATA(e))->weight)
+/// @}
 
 #define ag_xget(x,a) agxget(x,a)
 #define SET_RANKDIR(g,rd) (GD_rankdir2(g) = rd)
+/// @ingroup cgraph_edge
 #define agfindedge(g,t,h) (agedge(g,t,h,NULL,0))
+/// @ingroup cgraph_node
 #define agfindnode(g,n) (agnode(g,n,0))
-#define agfindgraphattr(g,a) (agattr(g,AGRAPH,a,NULL))
-#define agfindnodeattr(g,a) (agattr(g,AGNODE,a,NULL))
-#define agfindedgeattr(g,a) (agattr(g,AGEDGE,a,NULL))
+/// @ingroup cgraph_graph
+#define agfindgraphattr(g,a) (agattr_text(g,AGRAPH,a,NULL))
+/// @ingroup cgraph_node
+#define agfindnodeattr(g,a) (agattr_text(g,AGNODE,a,NULL))
+/// @ingroup cgraph_edge
+#define agfindedgeattr(g,a) (agattr_text(g,AGEDGE,a,NULL))
 
     typedef struct {
 	int flags;
@@ -619,4 +623,4 @@ typedef enum {NATIVEFONTS,PSFONTS,SVGFONTS} fontname_kind;
 #ifdef __cplusplus
 }
 #endif
-#endif
+/// @defgroup public_apis Graphviz public APIs
